@@ -31,9 +31,8 @@ from zope.intid.interfaces import IIntIds
 from zope.keyreference.interfaces import IKeyReference
 from zope.app.testing import setup
 from zope.component import getUtility, provideAdapter, provideUtility
-from zope.component import adapter
 from zope.container import btree
-from zope.interface import implements, alsoProvides
+from zope.interface import implements, alsoProvides, classImplements
 from zope.location.interfaces import ILocation
 from zope.location.location import locate
 from zope.publisher.browser import TestRequest
@@ -70,23 +69,28 @@ from schooltool.term.term import Term, getSchoolYearForTerm
 
 from schooltool.gradebook.activity import (Worksheet, Activity,
     getSectionActivities)
-from schooltool.gradebook.gradebook import Gradebook, getWorksheetSection 
-from schooltool.gradebook.attendance import getSectionAttendanceData
+from schooltool.gradebook.gradebook import (
+    getActivityScore,
+    getLinkedActivityScore,
+    getWorksheetAverageScore,
+    )
+from schooltool.gradebook.gradebook import (Gradebook, getWorksheetSection,
+    getGradebookSection)
 from schooltool.gradebook.gradebook_init import (setUpGradebookRoot, 
     getGradebookRoot, ReportLayout, ReportColumn, OutlineActivity)
 from schooltool.gradebook.interfaces import (IGradebookRoot, IGradebook,
-    IActivities, IWorksheet, ISectionAttendanceData)
-from schooltool.lyceum.journal.interfaces import ISectionJournalData
-from schooltool.lyceum.journal.journal import (LyceumJournalContainer,
-    getSectionJournalData, getSectionForSectionJournalData)
+    IActivities, IWorksheet, ISectionJournalData)
+from schooltool.lyceum.journal.journal import LyceumJournalContainer
 from schooltool.gradebook.browser.pdf_views import (StudentReportCardPDFView,
     GroupReportCardPDFView, StudentDetailPDFView, GroupDetailPDFView,
     FailingReportPDFView, AbsencesByDayPDFView, SectionAbsencesPDFView,
     GradebookPDFView)
 from schooltool.requirement.evaluation import Evaluation, getEvaluations
-from schooltool.requirement.interfaces import IEvaluations
+from schooltool.requirement.interfaces import (IScoreSystemContainer,
+    IEvaluations, IHaveEvaluations)
 from schooltool.requirement.scoresystem import (AmericanLetterScoreSystem,
-    DiscreteScoreSystemsVocabulary, DiscreteValuesScoreSystem)
+    DiscreteScoreSystemsVocabulary, DiscreteValuesScoreSystem,
+    getScoreSystemContainer, ScoreSystemAppStartup)
 
 
 BEGIN_2009 = datetime.date(datetime(2009, 1, 1))
@@ -178,6 +182,7 @@ class ApplicationStub(btree.BTreeContainer):
         int_ids.register(self.term)
 
         setUpGradebookRoot(self)
+        ScoreSystemAppStartup(self)()
 
 
 class DateManagerStub(object):
@@ -229,7 +234,7 @@ def setupSections(app):
     evaluation = Evaluation(activity2, maxss, 'C', thoffman)
     evaluations.addEvaluation(evaluation)
 
-    jd = ISectionAttendanceData(section1)
+    jd = ISectionJournalData(section1)
     calendar = ISchoolToolCalendar(section1)
     meeting = MeetingStub()
     meeting.unique_id = "unique-id-2009-01-01"
@@ -502,14 +507,16 @@ def doctest_GradebookPDFView():
           'shortTitle': 'Activ'}]
 
         >>> pprint(view.table())
-        [{'average': u'14.3%',
+        [{'absences': '',
+          'average': '14.3%',
           'grades': [{'activity': 'Activity', 'editable': True, 'value': 'F'},
                      {'activity': 'Activity-2', 'editable': True, 'value': 'C'}],
           'student': {'gradeurl': '...',
                       'id': 'aelkner',
                       'title': 'Elkner, Alan',
                       'url': 'http://localhost/persons/aelkner'},
-          'total': u'1.0'}]
+          'tardies': '',
+          'total': '1.0'}]
     """
 
 
@@ -524,16 +531,24 @@ def pdfSetUp(test=None):
     provideAdapter(getGroupContainer, [ISchoolYear], provides=IGroupContainer)
 
     provideAdapter(PersonLearnerAdapter, [IBasicPerson], provides=ILearner)
+    classImplements(BasicPerson, IHaveEvaluations)
     provideAdapter(getEvaluations, [IBasicPerson], provides=IEvaluations)
 
     provideAdapter(getTermForSectionContainer, [ISectionContainer],
                    provides=ITerm)
     provideAdapter(getSectionActivities, [ISection], provides=IActivities)
     provideAdapter(getTermForSection, [ISection], provides=ITerm)
+
+    from schooltool.gradebook.interfaces import ISectionJournalData
+    from schooltool.gradebook.journal import getSectionJournalData
+    provideAdapter(getSectionJournalData, [ISection],
+                   provides=ISectionJournalData)
+
+    from schooltool.lyceum.journal.interfaces import ISectionJournalData
+    from schooltool.lyceum.journal.journal import getSectionJournalData
+    from schooltool.lyceum.journal.journal import getSectionForSectionJournalData
     provideAdapter(getSectionJournalData, [ISection],
                    provides=ISectionJournalData)
-    provideAdapter(getSectionAttendanceData, [ISection],
-                   provides=ISectionAttendanceData)
     provideAdapter(getSectionForSectionJournalData, [ISectionJournalData],
                    provides=ISection)
 
@@ -541,9 +556,17 @@ def pdfSetUp(test=None):
                    provides=IGradebookRoot)
     provideAdapter(Gradebook, [IWorksheet], provides=IGradebook)
     provideAdapter(getWorksheetSection, [IWorksheet], provides=ISection)
+    provideAdapter(getGradebookSection, [IGradebook], provides=ISection)
     provideAdapter(getSchoolYearContainer, [ISchoolToolApplication], 
                    provides=ISchoolYearContainer)
 
+    provideAdapter(getActivityScore)
+    provideAdapter(getLinkedActivityScore)
+    provideAdapter(getWorksheetAverageScore)
+
+    provideAdapter(getScoreSystemContainer, [ISchoolToolApplication],
+                   provides=IScoreSystemContainer)
+
     provideAdapter(StupidKeyReference, [object], IKeyReference)
     provideAdapter(FakeURL, [ISchoolToolApplication, IBrowserRequest],
                    provides=IAbsoluteURL)
